﻿app.directive('bindHtmlUnsafe', function ($compile) {
    return function ($scope, $element, $attrs) {

        var compile = function (newHTML) { // Create re-useable compile function
            newHTML = $compile(newHTML)($scope); // Compile html
            $element.html('').append(newHTML); // Clear and append it
        };

        var htmlName = $attrs.bindHtmlUnsafe; // Get the name of the variable 
        // Where the HTML is stored

        $scope.$watch(htmlName, function (newHTML) { // Watch for changes to 
            // the HTML
            if (!newHTML) return;
            compile(newHTML);   // Compile it
        });

    };
});

app.controller("privacyAgreeCtrl", ['$scope', 'commonService', 'Session', 'UIService', function ($scope, commonService, Session, UIService) {
    var chessDb = null;

    //#### Don't Change
    var privacy = "ChessConnectPrivacyPolicy"
    var privacyYesResponse = "YES";

    //$scope.showPrivacyDetail = "block";
    $scope.privacyContent = "";

    $scope.IsChecked = false;

    $scope.redAlert = "";
    $scope.showNoConnection = "none";
    $scope.showPrivacyDetail = "none";
    //#### Don't Change

    try {
        if (isAndroidApp) {
            chessDb = window.sqlitePlugin.openDatabase({ name: "ChessConnect.db", location: 'default' });
            //alert("Android = Created Chess DB");
        }
        else {
            chessDb = window.openDatabase('ChessConnect', "0.1", "ChessConnect", 200000);
            //alert("WEB SQL = Created Chess DB");
        }
    }
    catch (e) {
        alert(e);
    }

    if (chessDb == null) {
        alert("Not able to access the Storage.");
        return;
    }

    $scope.getPrivacyDetail = function (title) {
        $scope.redAlert = "";

        UIService.showLoader('main');
        var promise = commonService.getCMSDetail(title);

        promise.then(function (data) {
            UIService.hideLoader('main');

            if (data != null) {
                //alert(data.IsSuccess);
                if (data.IsSuccess && data.CMSDetail != null) {
                    $scope.showNoConnection = "none";
                    $scope.showPrivacyDetail = "block";

                    $scope.privacyContent = data.CMSDetail.Content;
                }
                else {
                    $scope.showNoConnection = "block";
                    $scope.showPrivacyDetail = "none";

                    $scope.redAlert = "No data found.";
                }
            }
            else {
                $scope.showNoConnection = "block";
                $scope.showPrivacyDetail = "none";
                $scope.redAlert = "Check Internet Connectivity..";
            }
        });
    }

    $scope.NavigateToLogin = function () {
        location.href = "#/login";
        return;
    }

    $scope.UpdatePrivacy = function (intent, intentOption) {

        if (chessDb == null) {
            location.href = "#/login";
            return;
        }

        chessDb.transaction(function (transaction) {

            transaction.executeSql('Select * From SystemDetail Where Intent=?', [intent],
                function (tx, result) {
                    var len = result.rows.length
                    if (len > 0) {

                        transaction.executeSql('Update SystemDetail Set IntentContent=? Where Intent=?',
                            [intentOption, intent],
                            function (tx, results) {
                                $scope.NavigateToLogin();
                                return;
                            },
                            function (error) {
                                alert("Err - UPDPRIVACY");
                            });

                    }
                    else {

                        transaction.executeSql('Insert Into SystemDetail(Intent,IntentContent) Values (?,?)',
                            [intent, intentOption],
                            function (tx, results) {
                                $scope.NavigateToLogin();
                                return;
                            },
                            function (error) {
                                alert("Err - INSPRIVACY");
                            });
                    }
                },
                function (error) {
                    alert("Err - SUBPRIVACY");
                });
        });
    }

    $scope.agreeAndContinue = function () {
        //Validate and Agree Privacy

        if (!$scope.IAgreed) {
            alert("Please agree with Privacy Policy");
            return;
        }
        else {

            //alert("Checkbox is Checked");
            $scope.UpdatePrivacy(privacy, privacyYesResponse);
        }
    }

    $scope.CheckSystemDetail = function (intent) {
        chessDb.transaction(function (transaction) {
            transaction.executeSql('Select * From SystemDetail Where Intent=?', [intent],
                function (tx, result) {
                    var len = result.rows.length
                    //alert("Record Count = " + len);

                    if (len > 0) {
                        //isValuePresentInDb = true;

                        var curIntent = result.rows.item(0).Intent;
                        var curIntentContent = result.rows.item(0).IntentContent;

                        if (intent == curIntent && curIntentContent == privacyYesResponse) {
                            // Navigate to Login.

                            //alert("SystemDetail - Navigate To Login");
                            $scope.NavigateToLogin();
                            return;
                        }
                        else {

                            //alert("GET PRIVACY1");
                            $scope.getPrivacyDetail(privacy);
                            // Show Privacy Screen and Get Approve.
                        }
                    }
                    else {

                        //alert("GET PRIVACY2");
                        $scope.getPrivacyDetail(privacy);
                        // Show Privacy Screen and Get Approve.
                    }
                },
                function (error) {
                    alert("ERR-CHKSYSDB");
                    // alert("Error = CheckSystemDetail");
                });
        });
    }

    $scope.CreateSystemDetail = function () {
        chessDb.transaction(function (transaction) {
            transaction.executeSql('CREATE TABLE IF NOT EXISTS SystemDetail (DetailId integer primary key, Intent text, IntentContent text)', [],
                function (tx, result) {
                    //alert("SystemDetail - Created");

                    $scope.CheckSystemDetail(privacy);
                },
                function (error) {
                    alert("ERR - SYSDB");
                });
        });
    }

    if (chessDb != null) {
        $scope.CreateSystemDetail();
    }
    else {
        $scope.getPrivacyDetail(privacy);
    }
}]);
